///////////////////////////////////////////////////////////
/////////////Classe de Volume Rendering : VolRender////////
///////////////////Cyril Crassin Octobre 2003//////////////
///////////////////////////////////////////////////////////

#ifndef VolRender_h
#define VolRender_h

#ifdef WIN32
#include <windows.h>
#endif

#include "Vector.h"


#include <GL/gl.h>
#include <GL/glu.h>
//#include <GL/glext.h>
#include <glux/glux.h>


#include "Cg/cg.h"
#include <Cg/cgGL.h>


enum {X, Y, Z, W};
enum {R, G, B, A};



class VolRender{
	IVector fieldSize;

	//champ de scalaires
	float *tex3ddata;
	bool my3ddata;
	GLuint texID;
	char nbComp;

	//gradients
	float *gradField;
	bool gradEnable;
	GLuint gradID;

	//cuting planes
	bool cut;
	GLint cutbias;

//Params de controle
	GLfloat objangle[2];

	int slices;


	int planesList;

//////////////////////////////////////////
//Extension 3D texture, devenue standard mais Windows ne supportant que OpenGL 1.1... 
/*	PFNGLTEXIMAGE3DEXTPROC glTexImage3DEXT;
	PFNGLTEXSUBIMAGE3DEXTPROC glTexSubImage3DEXT;

	PFNGLACTIVETEXTUREARBPROC	glActiveTextureARB;

	PFNGLMULTITEXCOORD1FARBPROC	glMultiTexCoord1fARB;
	PFNGLMULTITEXCOORD2FARBPROC	glMultiTexCoord2fARB;
	PFNGLMULTITEXCOORD3FARBPROC	glMultiTexCoord3fARB;
	PFNGLMULTITEXCOORD4FARBPROC	glMultiTexCoord4fARB;
*/

	////////Cg///////
	CGcontext shaderContext;

	CGprofile vertexProfile;
	CGprofile fragmentProfile;

	CGprogram vertexProgram;
	CGprogram fragmentProgram;

	CGparameter cgParam_lightPos;
	/////////////////


	void CheckCgError();
	int initCg();
	GLuint tableID;


	void drawCube();

	void buildPlanes();
	
	void updateDataTexture();
	

public:
	VolRender(){
		tex3ddata=0; my3ddata=0;

		glGenTextures(1, &texID);
		cut=0;
		cutbias = 50;
		objangle[0]=0; objangle[1]=0;

		slices=200;

		gradField=0;

		fieldSize=IVector(128,128,128);
		nbComp=1;
	}

	virtual ~VolRender(){}

	int initExts();

	int init();

	void draw0();
	void draw();


	//Attachement de champs
	void bindScalarField(float *sf, IVector size);
	void bindRGBAField(float *sf, IVector size);

	//Mise a jour d'une partie du volume
	void updateDataTextureRegion(float *data, IVector pos, IVector size);

	//Chargement de donnes Raw
	void loadRawData(char *fich, IVector size, int nbcomp);
	//void loadRawDataOriginal(char *fich, IVector size, int nbcomp);
	void loadRawDataPart(char *fich, IVector size, int nbcomp, IVector partPos, IVector partSize, IVector fsize=IVector(-1,-1,-1));

	//Calcul du Gradient pour l'clairage
	void computeGradient();

	//Controle provisoire
	void setRotation(float x, float y){
		objangle[0]=x; objangle[1]=y;
	}
	void setSlices(int nb){
		slices=nb;
		buildPlanes();
	}
	int getSlices(){
		return slices;
	}

	void setCutBias(int nb){
		cutbias=nb;
	}
	void setCut(bool enable){
		cut=enable;
	}

	bool getCut(){ return cut; }


	void setLightPos(FVector pos){
		float tmp[]={pos.x, pos.y, pos.z};
		cgGLSetParameter3fv( cgParam_lightPos, tmp );
	}

	void setPalette(float *pal);

/*	int setResolution(IVector r){
		if(!my3ddata)
			return -2;

		if(r==fieldSize)
			return -1;

		if(tex3ddata)
			delete [] tex3ddata;
		fieldSize=r;

		tex3ddata=new float[(fieldSize.x)*(fieldSize.y)*(fieldSize.z)*nbComp];
	}*/
	

////////New: gestion des erreurs par exceptions////////
	/// definit les exceptions ///
	class Error {
	public:
		char *errString;

		Error(const char *es){
			if(es){
				errString = new char [strlen(es)+1];
				strcpy(errString, es);

			}else{
				errString=new char[strlen("#Erreur indefinie#")+1];
				strcpy(errString, "#Erreur indefinie#");
			}
		}
	};

	class CgError {
	public:
		char *errString;
		char *context;

		CgError(const char *es, const char *c){
			if(es && c){
				errString = new char [strlen(es)+1];
				strcpy(errString, es);
				context=new char [strlen(c)+1];
				strcpy(context, c);
			}else{
				errString=new char[strlen("#Erreur indefinie#")+1];
				strcpy(errString, "#Erreur indefinie#");

				context=new char[1];
				context[0]=0;

			}
		}
	};


};

#endif
