#include <stdlib.h>
#include <GL/glut.h>
#include <iostream.h>
#include <math.h>

const double rc = .1,ic = .1, jc = -.1, kc = 0; 
const int maxiter = 10;
const float limit = 2;

double slice = 1;

int wide=251, high=251;   // Global Variables for Length and Width
int winX=40, winY=40;     // X and Y positions for window
double cx = 0, cy = 0;
double vy = 0, vx = 0, dt = .1, ax = 0, ay = -9.8, v, theta, ar = .01;
float orthoxmin, orthoxmax, orthoymin, orthoymax, orthozmin, orthozmax;
int render1 = 0, render2 = 1;


class complex
{ public:
  double real, i, j, k;
  complex square();
  int numwithin();
  void iter();
  complex & operator = (complex & lhs);
};

complex complex::square()
{
  double a, b, c, d;
  a = real; b=i; c=j; d=k;
  complex ans;
  ans.real = a*a - b*b - c*c - d*d;
  ans.i = 2*a*b;
  ans.j = 2*a*c;
  ans.k = 2*a*d;
  return ans;
}

int complex::numwithin()
{
  double length;
  int m = 0;
  length = real*real + i*i + j*j + k*k;
  //  cout<<real<<' '<<i<<' '<<j<<' '<<k<<' '<<endl;
  while ((m<maxiter) && (length < limit*limit)) // limit IS THE BAILOUT CONDITION
  {
    iter();
    m++;
    length= real*real + i*i + j*j + k*k;
  }
  return m;
}

int niter(double x, double y, double z)
{
  double w = slice;
  complex temp; int len;
  temp.real = x;
  temp.i = y;
  temp.j = z;
  temp.k = w;
  len = temp.numwithin();
  return len;
}

complex & complex::operator = (complex & lhs)
{
  real = lhs.real;
  i = lhs.i;
  j = lhs.j;
  k = lhs.k;
  return (*this);
}


void complex::iter()
{
  complex temp = square();
  real = temp.real + rc;
  i = temp.i + ic;
  j = temp.j + jc;
  k = temp.k + kc;
}

void myinit(void)
{ 
  //glClearColor(0.5, 0.5, 0.5, 0.0);
  //glMatrixMode(GL_PROJECTION);
  //glLoadIdentity();

  GLfloat mat_specular[] = {1, 1, 1, 1.};
  GLfloat mat_shininess[] = {30.0};
  GLfloat light_position[] = {1, 1, -50, 0};
  GLfloat light_diffuse[] = {.8, .8, .8, 1};
  GLfloat light_ambient[] = {.1, .1, .1, 1};

  glShadeModel(GL_SMOOTH);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

  glClearColor(.4, 0.5, 0.1, 1.0); 

  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);

  glColorMaterial(GL_FRONT, GL_DIFFUSE);

  glMatrixMode(GL_PROJECTION);

  gluPerspective(60,        high  /       wide , 1, 800);
  gluLookAt(-20,-20,-20, 0, 0, 0, 0, 1, 0);


  /*
    orthoxmin = -limit;
    orthoxmax = limit;
    orthoymin = -limit;
    orthoymax = limit;
    orthozmin = -limit;
    orthozmax = limit;
    glOrtho(orthoxmin, orthoxmax, orthoymin, orthoymax, orthozmin, orthozmax);
  */
}



void display (void)
{

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  double x, y, z;
  glBegin(GL_POINTS);
  double xstep = 2*limit/(wide-1), ystep=2*limit/(high-1), zstep = 2*limit/(high-1), length;
    for(z = -limit; z < limit; z += zstep)
  // z = .2;
    for(y = -limit; y < limit; y += ystep)
      for(x = -limit; x < limit; x += xstep)
	{
	  length = niter(x, y, z);
	  if (length>4)
	    {
	      glColor3f(length/maxiter, (length/maxiter*2*3.14), sin(length/maxiter*2*3.14159));
	      glVertex3f(x, y, z);
	    }
	}
  glEnd;

  glutSwapBuffers();
}

int main(int argc, char* argv[])
{  
  glutInit(&argc, argv);  
  glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
  glutInitWindowPosition(winX,winY);
  glutInitWindowSize(wide, high);
  glutCreateWindow("Fractal");      
  myinit();                   
  glutDisplayFunc(display);     
  //  glutReDisplay(redisplay);
  glutMainLoop();               
  return 0;
}



/*
How to square a Quaternion:
(a, bi, cj, dk)^2.real         = a*a - b*b - c*c - d*d
(a, bi, cj, dk)^2.imaginary(i) = 2*a*b
(a, bi, cj, dk)^2.imaginary(j) = 2*a*c
(a, bi, cj, dk)^2.imaginary(k) = 2*a*d

*/

